package schelling;

import javax.swing.*; 
import java.awt.*;
import java.awt.event.*;

/*
 * @author rholguin
 *
 * This is the main panel that will be dispalyed to the user.  It will show the the before and after grid visual
 * it will also give them a user interface to allow them to adjust the parameters to whatever they like and
 * re-run the simulation
 */
public class MainPanel extends JPanel implements ActionListener, Runnable {
    
	private JLabel greenTol;
	private TextField greenText;
	
	private JLabel redTol; 
	private TextField redText;
	
	private JLabel iterations; 
	private TextField iterationText;
	
	private JLabel redMoves;
	private JLabel greenMoves;
	
	private JLabel refreshLabel;
	private TextField refreshText;
	
	private JLabel beforeLabel;
	private JLabel afterLabel;
	private JLabel happyBeforeRed;
	private JLabel happyAfterRed;
	private JLabel happyBeforeGreen;
	private JLabel happyAfterGreen;
	
	private JButton update;
	private JButton quit;
	
	private int redValue = 50;
	private int greenValue = 50;
	private int iterationValue = 1000;
	private int refreshValue = 10;
	private boolean runSim = true;
	
	private MyDialog dialog;
	private JFrame frame;
	private ReadFile file;
	private Simulation sim;
	
	private CreateGrid before;
	private CreateGrid after;
	
	private RandomGrid ranGrid;
	
	public MainPanel(JFrame f, boolean random, Schelling schel) { 
		setLayout(null);
		setSize(765, 690);
		
		frame = f;
		
		//create components to be displayed
		beforeLabel = new JLabel("Before:");
		afterLabel = new JLabel("After:");
		
		greenTol = new JLabel("Green Tolerance (%)");
		greenText = new TextField(""+greenValue,2);
		
		redTol = new JLabel("Red Tolerance (%)");
		redText = new TextField(""+redValue,2);
		
		iterations = new JLabel("How many Iterations?");
		iterationText = new TextField(""+iterationValue);
		
		refreshLabel = new JLabel("Refresh every                  Iterations");
		refreshText = new TextField("10");
		
		update = new JButton("Update");
		update.addActionListener(this);
		
		quit = new JButton("Quit Simulation");
		quit.setName("quit");
		quit.addActionListener(this);
		
		/*
		 * If not a random generated grid then use the first IF and get the grid from the file and create the simulation
		 * and before grid visual
		 * If it its random then created a random grid, create simulation and create before grid visual
		 */
		if(!random) {
			file = new ReadFile();
			if(file.getError() != null)
				Error(file.getError());
			before = new CreateGrid(file.getGrid());
			sim = new Simulation(file.getGrid(),redValue, greenValue);
		}
		else {
			ranGrid = new RandomGrid(schel.getRanParams()[0],schel.getRanParams()[1],schel.getRanParams()[2]);
			before = new CreateGrid(ranGrid.getGrid());
			sim = new Simulation(ranGrid.getGrid(), redValue, greenValue);
		}
		
		//initialize the simulation
		sim.initSim(redValue, greenValue);
		
		//create labels that are dependent on teh simulation being created first
		happyBeforeGreen = new JLabel("Happy Green Squares: "+ sim.getBeforeHappyLevel()[0]);
		happyBeforeRed = new JLabel("Happy Red Squares: "+ sim.getBeforeHappyLevel()[1]);
		happyAfterGreen = new JLabel("Happy Green Squares: "+ sim.getAfterHappyLevel()[0]);
		happyAfterRed = new JLabel("Happy Red Squares: "+ sim.getAfterHappyLevel()[1]);
		
		redMoves = new JLabel("Red Moves = "+sim.getRedMoves());
		greenMoves = new JLabel("Green Moves = "+sim.getGreenMoves());
		
		//create after grid visual
		after = new CreateGrid(sim.getNewGrid());
		
		removeAll();
		
		//add components to teh panel and set thier locations
		add(happyBeforeRed);
		add(happyBeforeGreen);
		add(refreshLabel);
		add(refreshText);
		add(afterLabel);
		add(beforeLabel);
		add(iterations);
		add(iterationText);
		add(greenTol);
		add(greenText);
		add(redTol);
		add(redText);
		add(update);
		add(quit);
		add(before);
		add(after);
		add(greenMoves);
		add(redMoves);
		add(happyAfterRed);
		add(happyAfterGreen);
		
		happyAfterRed.setBounds(470,372,happyBeforeRed.getPreferredSize().width+10,happyBeforeRed.getPreferredSize().width);
		happyAfterGreen.setBounds(470,380,happyBeforeGreen.getPreferredSize().width+10,happyBeforeGreen.getPreferredSize().width);
		happyBeforeRed.setBounds(70,372,happyBeforeRed.getPreferredSize().width+10,happyBeforeRed.getPreferredSize().width);
		happyBeforeGreen.setBounds(70,380,happyBeforeGreen.getPreferredSize().width+10,happyBeforeGreen.getPreferredSize().width);
		
		refreshLabel.setBounds(250,80,refreshLabel.getPreferredSize().width,refreshLabel.getPreferredSize().height);
		refreshText.setBounds(335,80,40,20);
		before.setBounds(20,180,250,250);
		beforeLabel.setBounds(120,150,beforeLabel.getPreferredSize().width, beforeLabel.getPreferredSize().width);
		after.setBounds(430,180,250,250);
		afterLabel.setBounds(530,150,beforeLabel.getPreferredSize().width, beforeLabel.getPreferredSize().width);
		
		iterations.setBounds(250,50,iterations.getPreferredSize().width, iterations.getPreferredSize().height);
		iterationText.setBounds(380,50,50,20);
		greenText.setBounds(190,50,30,20);
		greenTol.setBounds(70,50,greenTol.getPreferredSize().width, greenTol.getPreferredSize().height);
		redTol.setBounds(450,50,redTol.getPreferredSize().width, redTol.getPreferredSize().height);
		redText.setBounds(560,50,30,20);
		update.setBounds(300,110,update.getPreferredSize().width,update.getPreferredSize().height);
		quit.setBounds(290,410,quit.getPreferredSize().width,quit.getPreferredSize().height);
		
		greenMoves.setBounds(300,250,greenMoves.getPreferredSize().width,greenMoves.getPreferredSize().height);
		redMoves.setBounds(300,300,greenMoves.getPreferredSize().width,greenMoves.getPreferredSize().height);
	}
	
	public void actionPerformed(ActionEvent e) {
		greenValue = new Integer(greenText.getText()).intValue();
		redValue = new Integer(redText.getText()).intValue();
		iterationValue = new Integer(iterationText.getText()).intValue();
		refreshValue = new Integer(refreshText.getText()).intValue();
		
		/*
		 * First 2 if statments are for error checking on invalid input
		 * the next IF is for catching the quit button press
		 * The last is when the update button is pressed.
		 */
		if(greenValue < 0 || redValue < 0 || greenValue > 100 || redValue > 100 ) {
			dialog = new MyDialog(frame,"Red and Green values must be between 0 and 100", false);
			dialog.setVisible(true);
		}
		else if(iterationValue <= 0) {
			dialog = new MyDialog(frame,"Iteration value must be greater than 0", false);
			dialog.setVisible(true);
		}
		else if(e.getActionCommand().equals("Quit Simulation")) {
			runSim = false;
		}
		else
		{
			sim.initSim(redValue, greenValue);
			runSim = true;
			(new Thread(this)).start();
		}
	}
	
	public void run() {
		int iterationCount = 0;
		int refreshCount = 0;
		
		/*
		 * count starting from 0 to the user inputed iteration value.  Have another counter for the refresh value
		 * that increments every iteration.  When the refresh counter equals the user inputted refresh value
		 * then update the after grid visual with a 1 second delay
		 */
		while(iterationCount < iterationValue && runSim) {
			if(refreshCount == refreshValue) {
				refreshCount = 0;
				refresh();
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				sim.run();
			}
			else {
				sim.run();
				refreshCount++;
			}
			iterationCount++;
		}
		refresh();
	}
	
	//create error pop-up message
	private void Error(String text) {
		MyDialog dialog = new MyDialog(frame, text, true);
		dialog.setVisible(true);
	}
	
	//refresh after grid and nesseccary labels
	private void refresh() {
		after.updateGrid(sim.getNewGrid());
		redMoves.setText("Red Moves = "+sim.getRedMoves());
		greenMoves.setText("Green Moves = "+sim.getGreenMoves());
		
		setHappyLabels();
	}
	
	//refresh nesseccary labels
	private void setHappyLabels() {
		happyAfterGreen.setText("Happy Green Squares: "+ sim.getAfterHappyLevel()[0]);
		happyAfterRed.setText("Happy Red Squares: "+ sim.getAfterHappyLevel()[1]);
	}
}
